%DENOISING_DEMO: Denoising demonstration based on the SURE-LET principle
%   applied to redundant transform-domain thresholding.
%
%   See also UWT_SURELET_denoise, BDCT_SURELET_denoise and 
%   DICT_SURELET_denoise.
%
%   Author: Florian Luisier
%   Biomedical Imaging Group, EPFL, Lausanne, Switzerland.
%   This software is downloadable at http://fluisier.webnode.com/.
%
%   References:
%   [1] T. Blu, F. Luisier, "The SURE-LET Approach to Image Denoising,"
%   IEEE Transactions on Image Processing, vol. 16, no. 11, pp. 2778-2786,
%   November 2007.
%   [2] F. Luisier, "The SURE-LET Approach to Image Denoising," Swiss
%   Federal Institute of Technology Lausanne, EPFL Thesis no. 4566 (2010),
%   232 p., January 8, 2010.

clearvars;

path(path,'./images');
path(path,'./denoising');
path(path,'./transforms');
path(path,'./utilities');

% Load the original noise-free image
%-----------------------------------
filename = 'coco.tif';
original = double(imread(filename));

% Noise standard deviation
PSNR0 = 18;
sigma = 255*10^(-PSNR0/20);
% If DISPLAY = 'on', the original, noisy and denoised images are displayed
DISPLAY   = 'on';
% 'UWT' or 'BDCT' or 'DICT'
transform = 'UWT';    

% Size of the square DCT blocks (8, 12 or 16)
params.bsize = 12;
% Type of wavelet filter (default is 'haar')
params.wtype = 'haar';
% If boundary = 1 (default), mirror boundary conditions are applied
% If boundary = 0, periodic boundary conditions are applied
params.boundary = 1; 
% Number of BDCT radial frequency bands (default is 9)
params.nband = 9;                             
     
% Create input noisy image
%-------------------------
RandStream.setDefaultStream(RandStream('mt19937ar','seed',0));
noise = randn(size(original));
noise = sigma*noise/std(noise(:));
input = original+noise;        

% Denoise
%--------
start = clock;
switch lower(transform)
    case 'uwt'
        output = UWT_SURELET_denoise(input,params,sigma);
    case 'bdct'
        output = BDCT_SURELET_denoise(input,params,sigma);
    case 'dict'
        output = DICT_SURELET_denoise(input,params,sigma);
    otherwise
        disp('Unknown transform !');
        return;
end
time = etime(clock,start);       

% PSNR computation
%-----------------
MSE_0  = mean((input(:)-original(:)).^2);
MSE_D  = mean((output(:)-original(:)).^2);
PSNR_0 = 10*log10(255^2/MSE_0);
PSNR_D = 10*log10(255^2/MSE_D);

% Show PSNR results
%------------------
fprintf(['\nInput PSNR   : ' num2str(PSNR_0,'%.2f') '[dB]']);
fprintf(['\nOutput PSNR  : ' num2str(PSNR_D,'%.2f') '[dB]']);
fprintf(['\nElapsed time : ' num2str(time,'%.2f') '[s]\n\n']);

% Display results
%----------------
if(strcmp(DISPLAY,'on'))
    h = figure('Units','normalized','Position',[0 0.3 1 0.5]);
    set(h,'name','SURE-LET Denoising');
    subplot(1,3,1);imagesc(original,[0 255]);
    axis image;axis off;colormap gray(256);
    title('Original','fontsize',16,'fontweight','bold');drawnow;
    subplot(1,3,2);imagesc(input,[0 255]);
    axis image;axis off;colormap gray(256);
    title(['Noisy: PSNR = ' num2str(PSNR_0,'%.2f') ' dB'],'fontsize',16,...
    'fontweight','bold');drawnow;
    subplot(1,3,3);imagesc(output,[0 255]);
    axis image;axis off;colormap gray(256);
    title(['Denoised: PSNR = ' num2str(PSNR_D,'%.2f') ' dB'],'fontsize',...
    16,'fontweight','bold');drawnow;
end